/**
 * 
 */
package org.jboss.identity.idm.integration.jboss5;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.URL;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import javax.transaction.TransactionManager;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;
import org.hibernate.dialect.DialectFactory;
import org.hibernate.exception.SQLGrammarException;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.hibernate.tool.hbm2ddl.SchemaUpdate;
import org.jboss.identity.idm.api.IdentitySessionFactory;
import org.jboss.identity.idm.api.cfg.IdentityConfiguration;
import org.jboss.identity.idm.common.exception.IdentityException;
import org.jboss.identity.idm.common.transaction.TransactionManagerProvider;
import org.jboss.identity.idm.common.transaction.Transactions;
import org.jboss.identity.idm.impl.configuration.IdentityConfigurationImpl;
import org.jboss.mx.util.ObjectNameFactory;
import org.jboss.portal.common.io.IOTools;
import org.jboss.portal.common.net.URLTools;
import org.jboss.portal.common.util.CLResourceLoader;
import org.jboss.portal.common.util.LoaderResource;

import bsh.EvalError;
import bsh.Interpreter;


/**
 * Start the {@link IdentitySessionFactory}, and register it in the JNDI.
 * 
 * @author  Jeff Yu
 * @author  Boleslaw Dawidowicz
 * @author  Julien Viet  
 */

public class IDMService implements IDMServiceMBean {
	
	public final static String DEFAULT_JNDI = "java:/IdentitySessionFactory";
	
	private static Logger logger = Logger.getLogger(IDMService.class.getName());

   /** doCheck result - schema ok */
   private static final int RESULT_NONE = 0;

   /** doCheck result - schema need updates */
   private static final int RESULT_UPDATE = 1;

   /** doCheck result - schema not exist */
   private static final int RESULT_CREATE = 2;

   /** Hibernate config - if specified will be used to create SessionFactory*/
   private String hibernateConfigLocation;

   /** If hibernateConfigLocation option is present created SessionFactory will be registered in IdentityConfigurationRegistry
    * with this name*/
   private String hibernateSessionFactoryRegistryName;

   /** If hibernateConfigLocation option is present created SessionFactory will be registered in JNDI with this name*/
   private String hibernateSessionFactoryJNDIName;

   /** BSH script. If "doChecking" option is set and schema need to be created provided script will be invoked. This will
    * happen after IdentitySessionFactory is created. hibernateSessionFactory and identitySessionFactory object
    * are accessible in the script. */
   private String setupLocation;

   /** If true checks the schema existence on start and create it if necessary. */
   private boolean doChecking;

	private String idmConfigFile;
	
	private String idmSessionFactoryJNDI;
	
	private String SQLScript;
		
	private IdentitySessionFactory identitySessionFactory;

   private SessionFactory hibernateSessionFactory;

	private String exitSQL;
	
	private String datasource;
	
	private String objectName;
	
	private MBeanServer mbeanServer;

   /** The hibernate configuration object. */
   protected Configuration config;


   public IDMService(String idmConfigFile, MBeanServer mbeanServer) {
		this.idmConfigFile = idmConfigFile;
		this.mbeanServer = mbeanServer;
		this.idmSessionFactoryJNDI = DEFAULT_JNDI;
		this.objectName = IDMServiceMBean.defaultObjectName;
	}
	
	public String getIdmConfigFile() {
		return this.idmConfigFile;
	}

	public String getIdmSessionFactoryJNDI() {		
		return idmSessionFactoryJNDI;
	}

	public String getSQLScript() {
		return this.SQLScript;
	}

	public void setIdmSessionFactoryJNDI(String idmSessionFactoryJNDI) {
		this.idmSessionFactoryJNDI = idmSessionFactoryJNDI;
	}

	public void setSQLScript(String script) {
		SQLScript = script;
	}

   public String getHibernateConfigLocation()
   {
      return hibernateConfigLocation;
   }

   public void setHibernateConfigLocation(String hibernateConfigLocation)
   {
      this.hibernateConfigLocation = hibernateConfigLocation;
   }

   public String getSetupLocation()
   {
      return setupLocation;
   }

   public void setSetupLocation(String setupLocation)
   {
      this.setupLocation = setupLocation;
   }

   public boolean isDoChecking()
   {
      return doChecking;
   }

   public void setDoChecking(boolean doChecking)
   {
      this.doChecking = doChecking;
   }

   public String getHibernateSessionFactoryRegistryName()
   {
      return hibernateSessionFactoryRegistryName;
   }

   public void setHibernateSessionFactoryRegistryName(String hibernateSessionFactoryRegistryName)
   {
      this.hibernateSessionFactoryRegistryName = hibernateSessionFactoryRegistryName;
   }

   public String getHibernateSessionFactoryJNDIName()
   {
      return hibernateSessionFactoryJNDIName;
   }

   public void setHibernateSessionFactoryJNDIName(String hibernateSessionFactoryJNDIName)
   {
      this.hibernateSessionFactoryJNDIName = hibernateSessionFactoryJNDIName;
   }
   
   
   public String getObjectName() {
	   return objectName;
   }

   public void setObjectName(String objectName) {
	  this.objectName = objectName;
   }

   public void start() throws Exception {
		logger.info("Starting the JBoss Identity Management Service");
		if (needToInitializeDBFromScript()) {
			initializeDBFromScript();
		}

      InitialContext context = new InitialContext();

      final IdentityConfiguration identityConfiguration = new IdentityConfigurationImpl().configure(getIdmConfigFile());

      // If hibernate config is present than create SessionFactory manually and register it into identity config
      if (hibernateConfigLocation != null)
      {
         initializeDBFromHibernateConfiguration(context, identityConfiguration);
      }

      try {
         TransactionManager tm = TransactionManagerProvider.JBOSS_PROVIDER.getTransactionManager();
         Transactions.required(tm, new Transactions.Runnable()
         {
            public Object run() throws Exception
            {
               identitySessionFactory = identityConfiguration.buildIdentitySessionFactory();
               return null;
            }
         });
      } catch (Exception e) {

        throw new Exception("Cannot create IdentitySessionFactory", e); 
      }

      context.bind(getIdmSessionFactoryJNDI(), identitySessionFactory);
      mbeanServer.registerMBean(this, ObjectNameFactory.create(getObjectName()));
      
	   logger.info("Started the JBoss Identity Management Service");
	}

   
   private void initializeDBFromHibernateConfiguration(InitialContext context,
		   final IdentityConfiguration identityConfiguration) throws Exception, IdentityException {
	
	int check = RESULT_NONE;
	URL configURL = Thread.currentThread().getContextClassLoader().getResource(hibernateConfigLocation);
	 if (configURL == null)
	 {
	    throw new Exception("The config " + hibernateConfigLocation + " does not exist");
	 }

	 if (!URLTools.exists(configURL))
	 {
	    throw new Exception("The config " + configURL + " does not exist");
	 }

	 config = new AnnotationConfiguration().configure(configURL);

	 createHibernateSessionFactory();

	 if (getHibernateSessionFactoryJNDIName() != null)
	 {
	    try
	    {
	       context.bind(getHibernateSessionFactoryJNDIName(), hibernateSessionFactory);
	    }
	    catch (NamingException e)
	    {
	       e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
	    }
	 }

	 if (getHibernateSessionFactoryRegistryName() != null)
	 {
	    identityConfiguration.getIdentityConfigurationRegistry().
	       register(hibernateSessionFactory, getHibernateSessionFactoryRegistryName());
	 }

	 if (doChecking)
	 {
	    //check the schema
	    check = doCheck();
	    switch (check)
	    {
	       case RESULT_NONE:
	          break;
	       case RESULT_UPDATE:
	          updateSchema();
	          break;
	       case RESULT_CREATE:
	          createSchema();
	          createContent();

	          break;
	    }
	 }
}



   public void stop() throws Exception {
		logger.info("Stopping the JBoss Identity Management Service");		
	}
	
	public void destroy() throws Exception {

      InitialContext context = new InitialContext();

      try
      {
         if (identitySessionFactory != null)
         {
            identitySessionFactory.close();
            if (getIdmSessionFactoryJNDI() != null)
            {
               context.unbind(getIdmSessionFactoryJNDI());
            }
         }
      }
      catch (Exception e)
      {
         logger.severe(e.getMessage());
         e.printStackTrace();
      }

      if (hibernateSessionFactory != null)
      {
         try
         {
            if (getHibernateSessionFactoryJNDIName() != null)
            {
               context.unbind(getHibernateSessionFactoryJNDIName());

            }

            hibernateSessionFactory.close();
         }
         catch (Exception e)
         {
        	 logger.severe(e.getMessage());
            e.printStackTrace();  
         }
      }
      
      mbeanServer.unregisterMBean(ObjectNameFactory.create(getObjectName()));
	   logger.info("Stopped the JBoss Identity Management Service");
	}



	public String getExitSQL() {
		return exitSQL;
	}
	
	public void setExitSQL(String exitSQL) {
		this.exitSQL = exitSQL;
	}

	public String getDatasource() {
		return datasource;
	}

	public void setDatasource(String datasource) {
		this.datasource = datasource;
	}
	
	
	private boolean needToInitializeDBFromScript() {
		if (this.getDatasource() != null &&
				this.getSQLScript() != null &&
				this.getExitSQL() != null) {
			return true;
		}
		return false;
	}
	
	
	private void initializeDBFromScript() throws Exception {
		   DataSource ds = (DataSource)new InitialContext().lookup(datasource);
		   Connection conn = ds.getConnection();
		   boolean load = false;

		   Statement st = conn.createStatement();
		   ResultSet rs = null;
	      try
	      {
	         rs = st.executeQuery(exitSQL.trim());
	         rs.close();
	      }
	      catch (SQLException e)
	      {
	         load = true;
	      }
	      st.close();
	      if (!load)
	      {
	         logger.info(datasource + " datasource is already initialized");
	         return;
	      }

	      logger.info("Initializing " + datasource + " from listed sql files");

	      String[] list = SQLScript.split(",");
	      for (String sql : list)
	      {
	         executeSql(sql.trim(), conn);
	      }

	}
	
   private void executeSql(String resource, Connection conn) throws Exception
  {
		  URL url = Thread.currentThread().getContextClassLoader().getResource(resource);
		  String sql = new String(readStream(url.openStream()) , "UTF-8");
		  sql = sql.replaceAll("(?m)^SessionFactory --([^\n]+)?$", ""); // Remove all commented lines
		  final String[] statements ; 
		  statements = sql.split(";");
		  
		  for (String statement : statements)
		  {
			  if ((statement == null) || ("".equals(statement.trim()))) {
			  } else {
		         Statement sqlStatement = conn.createStatement();
		         try
		         {
		            sqlStatement.executeUpdate(statement);
		         }
		         finally
		         {
		            sqlStatement.close();
		         }
			  }
		  }
		}
   
   
	private byte[] readStream(InputStream stream) {
		if(stream == null) {
			throw new IllegalArgumentException("null 'stream' arg passed in method call.");
		}
		
		ByteArrayOutputStream outBuffer = new ByteArrayOutputStream();
		byte[] buffer = new byte[256];
		int readCount = 0;
		
		try {
			while((readCount = stream.read(buffer)) != -1) {
				outBuffer.write(buffer, 0, readCount);
			}
		} catch (IOException e) {
			throw new IllegalStateException("Error reading stream.", e);
		}		
		
		return outBuffer.toByteArray();
	}


   private void createHibernateSessionFactory() throws Exception
   {

      // Force transaction manager lookup class and JTA env
//      setPropertyIfAbsent("transaction.auto_close_session", "true");
//      setPropertyIfAbsent("transaction.flush_before_completion", "true");
//      setPropertyIfAbsent("hibernate.transaction.flush_before_completion", "true");
//      setPropertyIfAbsent("hibernate.transaction.factory_class", "org.hibernate.transaction.JTATransactionFactory");
//      setPropertyIfAbsent("hibernate.transaction.manager_lookup_class", "org.hibernate.transaction.JBossTransactionManagerLookup");

      // Set JNDI name if present and absent
      if (getHibernateSessionFactoryJNDIName() != null)
      {
         setPropertyIfAbsent("hibernate.session_factory_name", getHibernateSessionFactoryJNDIName());
      }

      // Detect the dialect if necessary
      String dialectName = config.getProperty(Environment.DIALECT);
      if (dialectName == null)
      {
         String dataSourceJNDI = config.getProperty(Environment.DATASOURCE);
         logger.fine("Detecting dialect with datasource " + dataSourceJNDI + " ...");
         DataSource ds = (DataSource)new InitialContext().lookup(dataSourceJNDI);
         Connection conn = null;
         try
         {
            conn = ds.getConnection();
            DatabaseMetaData meta = conn.getMetaData();
            String databaseName = meta.getDatabaseProductName();
            int databaseMajorVersion = getDatabaseMajorVersion(meta);
            dialectName = DialectFactory.determineDialect(databaseName, databaseMajorVersion).getClass().getName();
            config.setProperty(Environment.DIALECT, dialectName);
            logger.fine("Detected dialect " + dialectName + ", database is (" + databaseName + "," + databaseMajorVersion + ")");
         }
         finally
         {
            IOTools.safeClose(conn);
         }
      }
      logger.fine("Using dialect " + dialectName);
      if ("org.hibernate.dialect.HSQLDialect".equals(dialectName))
      {
         logger.info("You are using the file based HSQL database, this is not recommended on a production environment and will not work properly on a clustered environment.");
      }

      hibernateSessionFactory = config.buildSessionFactory();



   }



   private void setPropertyIfAbsent(String name, String value)
   {
      if (config.getProperty(name) == null)
      {
         config.setProperty(name, value);
      }
   }

   private int doCheck()
   {
      Session session = null;
      int numOfChecks = 0;
      int bad = 0;
      try
      {
         session = hibernateSessionFactory.openSession();
         Collection values = hibernateSessionFactory.getAllClassMetadata().values();
         numOfChecks = values.size();
         for (Iterator i = values.iterator(); i.hasNext();)
         {
            ClassMetadata cmd = (ClassMetadata)i.next();
            Query query = session.createQuery("from " + cmd.getEntityName());
            query.setFirstResult(0);
            query.setMaxResults(0);
            try
            {
               query.list();
            }
            catch (SQLGrammarException e)
            {
               // We consider that exception means that the schema does not exist
               bad++;
            }
         }
      }
      finally
      {
         IOTools.safeClose(session);
      }
      // There was no sql grammar exception - schema is ok!
      if (bad == 0)
      {
         logger.fine("The schema was checked as valid");
         //do nothing
         return RESULT_NONE;
      }
      // There is no existing valid schema;
      else if (bad == numOfChecks)
      {
         logger.fine("The schema was checked as not exists");
         // Totaly invalid schema
         return RESULT_CREATE;
      }
      // Schema is partialy corrupted
      else if (bad < numOfChecks)
      {
         // Schema needs updates;
         logger.fine("The schema was checked as need updates");
         return RESULT_UPDATE;
      }

      // If here something gone wrong...
      logger.fine("The schema was checked as need to be created");
      return RESULT_CREATE;
   }


   private void createSchema()
   {
      logger.fine("Creating database schema");
      try
      {
         SchemaExport export = new SchemaExport(config);
         export.create(false, true);
      }
      catch (Exception e)
      {
         logger.log(Level.SEVERE, "Cannot create schema:" , e);
      }
   }

   private void destroySchema()
   {
      logger.fine("Destroying database schema");
      try
      {
         SchemaExport export = new SchemaExport(config);
         export.drop(false, true);
      }
      catch (Exception e)
      {
         logger.log(Level.SEVERE, "Cannot destroy schema:", e);
      }
   }

   private void createContent()
   {

      LoaderResource setupResource = null;

      //
      if (setupLocation != null && setupResource == null)
      {
         setupResource = new CLResourceLoader().getResource(setupLocation);
      }

      if (setupResource != null)
      {
         if (setupResource.exists())
         {
            try
            {
               logger.info("Creating database content");
               final String script = setupResource.asString("UTF-8");

               // Create an interpreter and configures it
               final Interpreter interpreter = new Interpreter();
               interpreter.setClassLoader(Thread.currentThread().getContextClassLoader());
               interpreter.setOut(System.out);
               interpreter.set("SessionFactory", hibernateSessionFactory);

               TransactionManager tm = TransactionManagerProvider.JBOSS_PROVIDER.getTransactionManager();
               Transactions.required(tm, new Transactions.Runnable()
               {
                  public Object run() throws Exception
                  {
                     interpreter.eval(script);
                     return null;
                  }
               });
            }
            catch (EvalError e)
            {
               logger.log(Level.SEVERE, "Error in the bsh script:", e);
            }
            catch (IllegalStateException e)
            {
            	logger.log(Level.SEVERE, "Error in the bsh script:", e);
            }
            catch (Exception e)
            {
            	logger.log(Level.SEVERE, "Error in the bsh script:", e);
            }
         }
         else
         {
            logger.info("There is a setup URL but the not valid " + setupResource);
         }
      }
   }


   private void updateSchema()
   {
      logger.info("Updating database schema");
      SchemaUpdate update = new SchemaUpdate(config);
      update.execute(false, true);
   }


   private int getDatabaseMajorVersion(DatabaseMetaData meta)
   {
      try
      {
         Method gdbmvMethod = DatabaseMetaData.class.getMethod("getDatabaseMajorVersion", null);
         return ((Integer)gdbmvMethod.invoke(meta, null)).intValue();
      }
      catch (NoSuchMethodException nsme)
      {
         return 0;
      }
      catch (Throwable t)
      {
         logger.info("could not get database version from JDBC metadata");
         return 0;
      }
   }

}
